from BaseHTTPServer import (
    HTTPServer,
    BaseHTTPRequestHandler,
    )
import httplib
import os
from signal import SIGKILL
from StringIO import StringIO
from unittest import TestCase

from testtools import ExpectedException

from grackle.client import (
    GrackleClient,
    )


class ForkedFake:

    def __init__(self, port, messages=None):
        self.pid = None
        self.port = port
        self.messages = messages
        self.read_end, self.write_end = os.pipe()

    def is_ready(self):
        os.write(self.write_end, 'asdf')

    def __enter__(self):
        pid = os.fork()
        if pid == 0:
            self.start_server()
        self.pid = pid
        os.read(self.read_end, 1)
        return

    def start_server(self):
        service = HTTPServer(('', self.port), FakeGrackleRequestHandler)
        service.messages = self.messages
        self.is_ready()
        service.serve_forever()

    def __exit__(self, exc_type, exc_val, traceback):
        os.kill(self.pid, SIGKILL)


class FakeGrackleRequestHandler(BaseHTTPRequestHandler):

    def do_POST(self):
        message = self.rfile.read(int(self.headers['content-length']))
        if message == 'This is a message':
            self.send_response(httplib.CREATED)
            self.end_headers()
            self.wfile.close()
        else:
            self.send_error(httplib.BAD_REQUEST)


def fake_grackle_service(client, messages=None):
    if messages is None:
        messages = {}
    return ForkedFake(client.port, messages)


class TestPutMessage(TestCase):

    def test_put_message(self):
        client = GrackleClient('localhost', 8436)
        with fake_grackle_service(client):
            client.put_message('arch1', 'asdf', StringIO('This is a message'))
            with ExpectedException(Exception, 'wtf'):
                client.put_message('arch1', 'asdf',
                    StringIO('This is not a message'))


class TestGetMessages(TestCase):

    def test_get_messages(self):
        client = GrackleClient('localhost', 8435)
        with fake_grackle_service(client,
            {'baz':
            [{'message-id': 'foo'},
             {'message-id': 'bar'}]}):
            response = client.get_messages('baz')
        self.assertEqual(['bar', 'foo'], sorted(response.keys()))
